#!/usr/bin/env node


/**
* Engine version.
*/
var version = '0.0.1';
/**
 * Template Engine Server port
 */
var port;
var templatePath;


/**
* Usage documentation.
*/
var usage = ''
+ '\n'
+ ' Usage: template-engine.js [options]\n'
+ '\n'
+ ' Options:\n'
+ ' -v, --version output template engine version\n'
+ ' -h, --help output help information\n'
+ ' -c, --create create template engine server\n'
;

// Parse arguments
var args = process.argv.slice(2);
if(!args.length){
    abort(usage);
}
while (args.length) {
    var arg = args.shift();
    switch (arg) {
        case '-h':
        case '--help':
            abort(usage);
            break;
        case '-v':
        case '--version':
            abort(version);
            break;
        case '-cl':
            args.length ? (port = args.shift()) : abort(' -cl requires a port argument');
            createEngine();
            break;
        case '-c':
        case '--create':
            args.length ? (port = args.shift()) : abort(' -c requires a port argument');
            console.log = function(data){};
            createEngine();
            break;
        default:
            abort(usage);
    }
}


/**
* Create application at the given directory `path`.
*/
function createEngine() {
    
    var TemplateServer = require('../lib/TemplateServer.js').TemplateServer;
    var tsInstance = new TemplateServer();
    tsInstance.getClusterNode().listen(port);
    console.log('Server runing @ ' + port + '...');
    
}


/**
* Exit with the given `str`.
*
* @param {String} str
*/
function abort(str) {
    console.error(str);
    process.exit(1);
}
