var Class            = require("core").Class;
var Request          = require("core").Net.Request;
var Response         = require("core").Net.Response;
var RequestProcessor = require("core").Net.RequestProcessor;
var BinaryData       = require("core").Data.BinaryData;
var JsonRpcRequest   = require('core').JsonRpc.JsonRpcRequest;
var JsonRpcResponse  = require('core').JsonRpc.JsonRpcResponse;
var JsonRpcAbstract  = require('core').JsonRpc.JsonRpcAbstract;
var ErrorEvent       = require('core').Event.ErrorEvent;

var RenderingController = require("./RenderingController.js").RenderingController;


/**
 * @class TemplateRequestProcessor Base class for Request processor family.
 * @param {Request} request
 * @param {Response} response
 */
var TemplateRequestProcessor = function(){
    this.Extends = RequestProcessor;

    this._request = null;
    this._response = null;
    
    this._jsonRpcRequest = null;
    
    this.process = function(request, response){
        console.time('   -- REQUEST TIME');
        
        this._request = request;
        this._response = response;
        
        try {
            this._jsonRpcRequest = new JsonRpcRequest(request.getBody().toString());
        } catch(ex) {
            this._onError(new ErrorEvent(JsonRpcAbstract.Exceptions.INPROPER_JSON_STRING, null, -32000, ex));
            return;
        }
        
        try {
            var controller = new RenderingController(this._jsonRpcRequest.getParams());
            controller.addEventListener(RenderingController.RENDERED, this._onSuccess, this);
            controller.addEventListener(RenderingController.ERROR, this._onError, this);
            controller.render();
         } catch(e) {
            this._onError(e);
         }        
    };
    
    this._onSuccess = function(e){
        console.log('  -- TplRP COMPLETE. sending response...');

        var jsonRpcResponse = new JsonRpcResponse();
        jsonRpcResponse.setId(this._jsonRpcRequest.getId());
        
        jsonRpcResponse.setResult(e.data);

        this._setResponseHeaders();
        
        this._response.setBody(new BinaryData(jsonRpcResponse.toString(), BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
        this._response.send();
        console.timeEnd('   -- REQUEST TIME');
        console.log('-- request ended');
    };

    this._onError = function(e){
        console.log('  -- TplRP ERROR. sending error response...');

        if(typeof e ==='string'){
            console.log('     -- ' + e);
        } else {
            console.log('     -- ' + e.code + ': ' + e.message);
        }
       
        var jsonRpcResponse = new JsonRpcResponse();
        if(this._jsonRpcRequest){
            jsonRpcResponse.setId(this._jsonRpcRequest.getId());
        } else {
            jsonRpcResponse.setId('error');
        }
        
        if(typeof e ==='string'){
            jsonRpcResponse.setError(-32000, e, null);
        } else {
            jsonRpcResponse.setError(e.code, e.message, e.data);
        }

        this._setResponseHeaders();

        this._response.setBody(new BinaryData(jsonRpcResponse.toString(), BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
        this._response.send();
        console.timeEnd('   -- REQUEST TIME');
        console.log('-- request ended');
    };
    
    this._setResponseHeaders = function(){
        this._response.setStatusCode(200);
        this._response.setHeader('Content-Type', 'application/json; charset=utf-8');
        this._response.setHeader('Access-Control-Allow-Origin', '*');
    };
    
}
TemplateRequestProcessor = new Class( new TemplateRequestProcessor() );
exports.TemplateRequestProcessor = TemplateRequestProcessor;    