var Class = require("core").Class;
var fs = require('fs');

/**
 * @class Main resource cache controller
 * @requires Class
 * @requires fs
 *
 * @param {Object} configuration
 */
var Cache = function(){

    /**
     * @property {Object} _configuration
     * @private
     */
    this._configuration = null;
    

    /** @ignore */
    this.initialize = function(configuration){
        this._configuration = configuration;
    };

    /**
     * cache everything mensioned in configuraion using
     * special classes for given resource types
     */
    this.cache = function(){

        for(var i=0, max=this._configuration.length;i<max;i++){
            var configItem = this._configuration[i];

            var resourceLib = require('./resource/' + configItem.className);
            var ResourceClass = resourceLib[configItem.className];
            var resourceInstance = new ResourceClass(configItem);
            
            this._cacheRecursive(resourceInstance.getResourcePath(), resourceInstance);
        }
    };

    /**
     * cache recursive used by main Cache() method
     *
     * @private
     * @param {String} templatePath
     * @param {IResourceCache} resourceInstance
     */
    this._cacheRecursive = function(templatePath, resourceInstance){
        var templateFiles = fs.readdirSync(templatePath);
        var fileContents = '';
        var statInfo = null;

        for(var i = 0, l = templateFiles.length; i < l; i++){
           if(templateFiles[i]!==resourceInstance.getExcludePattern()){
               path = templatePath + templateFiles[i];
               statInfo = fs.statSync(path);
               if(statInfo.isDirectory()){
                   this._cacheRecursive(path+'/', resourceInstance);
               } else {
                   fileContents = fs.readFileSync(templatePath + templateFiles[i], 'utf8');
                   resourceInstance.cache(templatePath+templateFiles[i], fileContents);
               }
           }
        }
    };


};

Cache = new Class(new Cache());
exports.Cache = Cache;
