var Class = require("core").Class;

/**
 * @class Interface for classes that implements resource caching of any kind
 * @requires Class
 */
var IResourceCache = function(){

    /**
     * @property {Object} _configuration
     * @private
     */
    this._configuration = null;

    /** @ignore */
    this.initialize = function(configuration){
        this._configuration = configuration;
    };

    /**
     * returns path to resources
     * @returns {String}
     */
    this.getResourcePath = function(){
        return this._configuration.resourcePath;
    };

    /**
     * returns exclude resource regexp pattern
     * @returns {String}
     */
    this.getExcludePattern = function(){
        return this._configuration.excludePattern;
    };

    /**
     * abstract method. you need to override this method in your subclass
     * @throws {Exception} Not implemented
     */
    this.cache = function(){
        throw "Not Implemented. You must override this method";
    };
};

IResourceCache = new Class(new IResourceCache());
exports.IResourceCache = IResourceCache;