var Class           = require('core').Class;
var EventDispatcher = require('core').Event.EventDispatcher;

/**
 * Abstract class - direct instance cannot be created.
 * @class Providing simple interface of Driver's
 * @extends EventDispatcher
 * @throws {DriverAbstract.Exception.INITIALIZE_NOT_IMPLEMENTED}
 * @requires Class 
 * @requires EventDispatcher
 */
var DriverAbstract = function(){

    /** @ignore */
    this.Extends = EventDispatcher;
    
    /** @ignore */
    this.initialize = function(){
        throw DriverAbstract.Exception.INITIALIZE_NOT_IMPLEMENTED;  
    };
    /**
     * Iterface method. Should execute model loading.
     * @throws {DriverAbstract.Exception.LOAD_NOT_IMPLEMENTED}
     */
    this.load = function(){
        throw DriverAbstract.Exception.LOAD_NOT_IMPLEMENTED;
    };
};

DriverAbstract = new Class(new DriverAbstract());
/**
 * @constant
 * @static
 */
DriverAbstract.LOADED = "DriverAbstract_LOADED";
/**
 * @constant
 * @static
 */
DriverAbstract.ERROR = "DriverAbstract_ERROR";

/**
 * Namespace for exeptions messages.
 * @constant
 * @static
 * @namespace
 */
DriverAbstract.Exception = {};
/**
 * 
 * @constant
 * @static
 */
DriverAbstract.Exception.INITIALIZE_NOT_IMPLEMENTED = "Method::initialize() - not implemented!";
/**
 * @constant
 * @static
 */
DriverAbstract.Exception.LOAD_NOT_IMPLEMENTED = "Method::load() - not implemented!";

exports.DriverAbstract = DriverAbstract;