var Class           = require('core').Class;
var Event           = require('core').Event.Event;
var JsonRpcResponse = require('core').JsonRpc.JsonRpcResponse;

var DriverAbstract  = require('./DriverAbstract').DriverAbstract;


/**
 * Special implementation of driver which not making any remote or I/O calls. Given 'data' will be threated as retrived data after calling load method.
 * @class DriverLocal Special implementation of driver which not making any remote or I/O calls.
 * @extends DriverAbstract
 * @requires Class 
 * @requires DriverAbstract 
 * @requires Event
 * @param {Object} data Model data.
 * @throws {DriverLocal.Exception.NO_DATA_GIVEN} if given data is null.
 */
var DriverLocal = function(){

    /** @ignore */
    this.Extends = DriverAbstract;
    
    /**
     * @property {Object} _data
     * @private
     */
    this._data = null;

    /** @ignore */
    this.initialize = function(data){
        if(!data){
            throw DriverLocal.Exception.NO_DATA_GIVEN;
        }

        this._data = data;
    };

    /**
     * Implements DriverAbstract interface. Dispatches DriverAbstract.LOADED event with given model data.
     */
    this.load = function(){
        var response = new JsonRpcResponse();
        response.setResult(this._data);
        this.dispatchEvent(new Event(DriverAbstract.LOADED, response));
    };
};

DriverLocal = new Class(new DriverLocal());

/**
 * Namespace for exeptions messages.
 * @constant
 * @static
 * @namespace
 */
DriverLocal.Exception = {};
/**
 * @constant
 * @static
 */
DriverLocal.Exception.NO_DATA_GIVEN = "No data given!";

exports.DriverLocal = DriverLocal;