describe('DriverManager', function(){
    var Class = DriverManager = DriverAbstract = null;
    var TestDriver = null, FakeDriver = null;
    var manager = null;
    
    beforeEach(function(){
        this.addMatchers({
            toBeInstanceOf: function(expected) { 
                return (this.actual instanceof expected);
            }
        });
    });
    
    it('loading dependencies', function(){
        expect(function(){
            Class = require("../lib/mootools/mootools-node.js").Class;
        }).not.toThrow();
        expect(function(){
            DriverManager = require('../lib/DriverManager.js').DriverManager;
        }).not.toThrow();
        expect(function(){
            DriverAbstract = require('../lib/DriverAbstract.js').DriverAbstract;
        }).not.toThrow();
    });
    
    it('creating mocks', function(){
        expect(function(){
            TestDriver = function(){
                this.Extends = DriverAbstract;
                this.initialize = function(){
                };
            };
            TestDriver = new Class(new TestDriver());
        }).not.toThrow();
        
        expect(function(){
            FakeDriver = function(){
                this.initialize = function(){
                };
            };
            FakeDriver = new Class(new FakeDriver());
        }).not.toThrow();
    });

    it('addDriver - exceptions', function(){
        var manager = new DriverManager(), fakeDriver = new FakeDriver(), driver = new TestDriver();

        expect(function(){
            manager.addDriver(fakeDriver);
        }).toThrow(DriverManager.Exception.WRONG_INSTANCE);
        
        expect(function(){
            manager.addDriver(driver);
        }).not.toThrow(DriverManager.Exception.WRONG_INSTANCE);            
    });
    
    it('addDriver - adding same driver two times', function(){
        var manager = new DriverManager(), driver = new TestDriver();

        expect(manager.addDriver(driver)).toBeTruthy();
        expect(manager.addDriver(driver)).toBeFalsy();
    });

    it('createDriver', function(){
        var driver = null;

        expect(function(){
            driver = DriverManager.createDriver("DriverLocal", {});
        }).not.toThrow(DriverManager.Exception.WRONG_INSTANCE);
        expect(driver).toBeInstanceOf(DriverAbstract);
        
        expect(function(){
            driver = DriverManager.createDriver("Eloszka", {});
        }).toThrow(DriverManager.Exception.DRIVER_DOES_NOT_EXIST);
    });
    
    it('fetch', function(){
        var manager = new DriverManager();
        
        var driver1 = DriverManager.createDriver("DriverLocal", {'name':"jeden"});
        spyOn(driver1, 'load');
        
        var driver2 = DriverManager.createDriver("DriverLocal", {'name':"dwa"});
        spyOn(driver2, 'load');
        
        manager.addDriver(driver1);
        manager.addDriver(driver2);
        manager.fetch(); 
        
        expect(driver1.load).toHaveBeenCalled();
        expect(driver2.load).toHaveBeenCalled();
    });

});