describe('DriverRemote', function(){
    var Class = OpalLoader = EventDispatcher = Event = ErrorEvent = DriverRemote = DriverAbstract = null;
    var FakeOpalLoader = null;

    var testingOk = "testing ok", testingError = 'testing error';
    
    it('loading dependencies', function(){
        expect(function(){
            Class = require('../lib/mootools/mootools-node.js').Class;
        }).not.toThrow();
        expect(function(){
            OpalLoader = require('../lib/core/opal/OpalLoader.js').OpalLoader;
        }).not.toThrow();
        expect(function(){
            EventDispatcher = require('../lib/core/event/EventDispatcher.js').EventDispatcher;
        }).not.toThrow();
        expect(function(){
            Event = require('../lib/core/event/Event.js').Event;
        }).not.toThrow();
        expect(function(){
            ErrorEvent = require('../lib/core/event/ErrorEvent.js').ErrorEvent;
        }).not.toThrow();
        expect(function(){
            DriverRemote = require('../lib/DriverRemote.js').DriverRemote;
        }).not.toThrow();
        expect(function(){
            DriverAbstract = require('../lib/DriverAbstract.js').DriverAbstract;
        }).not.toThrow();
    });
    
    it('fake opal loader', function(){
        expect(function(){
            FakeOpalLoader = function(){
                this.Extends = EventDispatcher;

                this._mode;

                this.initialize = function(mode){
                    this._mode = mode;
                };
                
                this.load = function(){
                    if(typeof this._mode == "undefined" || this._mode == false){
                        this.dispatchEvent(new ErrorEvent(OpalLoader.ERROR, null, -1337, testingError));
                    }else{
                        this.dispatchEvent(new Event(OpalLoader.LOADED, testingOk));
                    }
                };
            };
            FakeOpalLoader = new Class(new FakeOpalLoader());
        }).not.toThrow();
    });

    it('creating', function(){
        expect(function(){ new DriverRemote(); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        expect(function(){ new DriverRemote(null); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        expect(function(){ new DriverRemote(undefined); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        expect(function(){ new DriverRemote(123); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        expect(function(){ new DriverRemote("eloszka"); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        expect(function(){ new DriverRemote([]); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        expect(function(){ new DriverRemote(new (function(){})); }).toThrow(DriverRemote.Exception.WRONG_PARAMS);
        
        expect(function(){ new DriverRemote({}); }).not.toThrow(DriverRemote.Exception.WRONG_PARAMS);
    });
    
    it('load', function(){
        var driver = new DriverRemote({});
        
        expect(function(){
            driver.load();
        }).toThrow(DriverRemote.Exception.OPAL_WRONG_PARAMS);
    });
    
    it('load success', function(){
        var responseEvent = null;
        
        runs(function(){
            var driver = new DriverRemote({url: "eloszka.pl", params: null, method: null});
            //FIXME: wiem ze jest to slabe ale nie wiem jak to zrobic lepiej
            driver._loader = new FakeOpalLoader(true);

            driver.addEventListener(DriverAbstract.LOADED, function(e){ responseEvent = e; });
            driver.addEventListener(DriverAbstract.ERROR, function(e){ responseEvent = e; });
            driver.load();
        });

        waitsFor(function(){
            return responseEvent != null;
        }, 1000);

        runs(function(){
            expect(responseEvent.type).toEqual(DriverAbstract.LOADED);
            expect(responseEvent.data).toEqual(testingOk);
        });
    });
    
    it('load success', function(){
        var responseEvent = null;
        
        runs(function(){
            var driver = new DriverRemote({url: "eloszka.pl", params: null, method: null});
            //FIXME: wiem ze jest to slabe ale nie wiem jak to zrobic lepiej
            driver._loader = new FakeOpalLoader(false);

            driver.addEventListener(DriverAbstract.LOADED, function(e){ responseEvent = e; });
            driver.addEventListener(DriverAbstract.ERROR, function(e){ responseEvent = e; });
            driver.load();
        });

        waitsFor(function(){
            return responseEvent != null;
        }, 1000);

        runs(function(){
            expect(responseEvent.type).toEqual(DriverAbstract.ERROR);
            //FIXME: walidowac wiadomosc z bledu, wymaga poprawek w DriverRemote::_onError
        });
    });
});