describe('RequestProcessor', function(){

    var RequestProcessor = null;
    var rp = null;
    var http = null;
    var serverPort = 8080;

    it('can load RequestProcessor Class', function(){
        expect(function(){
            RequestProcessor = require('../lib/RequestProcessor.js').RequestProcessor;
        }).not.toThrow();
    });

    it('can load http module', function(){
        expect(function(){
            http = require('http');
        }).not.toThrow();
    });

    it('proper properly init', function(){
        expect(function(){
            rp = new RequestProcessor();
        }).toThrow();
    });

    it('proper can properly init', function(){
        expect(function(){
            rp = new RequestProcessor({}, {});
        }).toThrow();
    });

    it('executing process with bad parameters', function(){
        expect(function(){
            rp = new RequestProcessor({}, {});
            rp.process();
        }).toThrow();
    });

    it('executing bad request ', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
        });
        
        waits(0);
        
        runs(function(){
            makeRequest('{"params":[]}');
        });

        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage).toBe('{"jsonrpc":"2.0","error":{"code":-32600,"message":"The received JSON is not a valid JSON-RPC Request."}}');
            server.close();
        });


    });


    it('executing proper request', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
        });

        waits(0);
        
        runs(function(){
            makeRequest('{"jsonrpc":"2.0","method":"render","params":[{"t001":{"template":"1.jqtpl","data": "Przykladowy tytul"}}],"id": "example"}');
        });

        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage).toBe('{"id":"example","jsonrpc":"2.0","result":[{"t001":{"_result":""}}]}');
            server.close();
        });


    });


    it('executing proper request', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){
                expect(function(){
                    var requestProcessor = new RequestProcessor(request, {});
                }).toThrow();
                server.close();
            });
            server.listen(serverPort, "localhost");
        });

        waits(0);

        runs(function(){
            makeRequest('{"jsonrpc":"2.0","method":"render","params":[{"t001":{"template":"1.jqtpl","data": "Przykladowy tytul"}}],"id": "example"}');
        });

        waits(1000);
        

    });


    it('making json parse request', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
            
        });

        waits(0);

        runs(function(){
            makeRequest('{{{{{{{{{{{');
        });

        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage.search("Invalid JSON. An error occurred on the server while parsing the JSON text")).toBeGreaterThan(-1);
            server.close();
        });


    });


    it('trying to use other than render method', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
            
        });

        waits(0);

        runs(function(){
            makeRequest('{"jsonrpc":"2.0","method":"NOT_______________RENDER","params":[{"t001":{"template":"1.jqtpl","data": "Przykladowy tytul"}}],"id": "example"}');
        });


        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage.search("The requested remote-procedure does not exist / is not available.")).toBeGreaterThan(-1);
            server.close();
        });


    });



    it('invalid parameters to render', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
            
        });

        waits(0);

        runs(function(){
            makeRequest('{"jsonrpc":"2.0","method":"render","params":"","id": "example"}');
        });

        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage.search("Invalid method parameters.")).toBeGreaterThan(-1);
            server.close();
        });


    });




    it('invalid driver specified', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
        });

        waits(0);

        runs(function(){
            makeRequest('{"jsonrpc":"2.0","method":"render","params":[        {            "t001": {                "template": "templates/template02.jqtpl",                "#data": {                    "driver": "DriverRemote!!",                    "url": "qi.ckm.onetapi.pl",                    "method": "get_asset_detail",                    "params": {                        "ID_Publikacji": "25822.434297834",                        "Service": "wiadomosci.onet.pl"                    }                }            }        }    ],"id": "example"}');
        });

        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage.search("Internal JSON-RPC error.")).toBeGreaterThan(-1);
            server.close();
        });


    });



    it('fatal exception test', function(){

        var returnMessage = null;
        var server = null;

        var makeRequest = function(postData){
            var req = http.request({
               host: 'localhost',
               port: serverPort,
               path: '/',
               method: 'POST'
            }, function(res) {
              //console.log('STATUS: ' + res.statusCode); console.log('HEADERS: ' + JSON.stringify(res.headers));
              var retData = [];
              res.setEncoding('utf8');
              res.on('data', function (chunk) {
                retData.push(chunk);
              });
              res.on('end', function(){
                returnMessage = retData.join("");
              });
            });

            req.on('error', function(e) {
                throw e.message;
            });

            // write data to request body
            req.write(postData);
            req.end();
        };

        runs(function(){
            server = http.createServer(function(request, response){

                var that = this;
                if (request.method == 'POST') {
                    var postData = [];
                    request.on('data', function(data) {
                        postData.push(data);
                    });
                    request.on('end', function(){
                        request.data = postData.join("");
                        var requestProcessor = new RequestProcessor(request, response);
                        requestProcessor.process();
                    });
                }

            });
            server.listen(serverPort, "localhost");
        });

        waits(0);

        runs(function(){
            makeRequest('{"jsonrpc":"2.0","method":"render","params":[],"id": "example"}');
        });

        waitsFor(function() {
          return returnMessage!==null;
        }, "response never delivered", 1000);

        runs(function(){
            expect(returnMessage.search("Empty template arguments")).toBeGreaterThan(-1);
            server.close();
        });


    });


});