describe("TemplateManager", function(){
    var configuration = {}, TemplateManager = JqTemplate = StylusTemplate = Template = JsonRpcAbstract = Class = TestTemplate = JsonRpcMock = null;
    
    beforeEach(function(){
        this.addMatchers({
           toBeInstanceOf: function(classObj){
               var test;

               if(this.actual instanceof Function){
                   test = this.actual();
               }else{
                   test = this.actual;
               }
               return (test instanceof classObj); } 
        });
        
        configuration = {
                template: _getConfigFromFile('TemplateManager/Template.json'),
                unresolved: _getConfigFromFile('TemplateManager/UnresolvedType.json')
            };        
    });
    
    it("requiring components", function(){
        expect(function(){
            TemplateManager = require('../lib/TemplateManager.js').TemplateManager;
        }).not.toThrow();

        expect(function(){
            JqTemplate = require('../lib/template/JqTemplate.js').JqTemplate;
        }).not.toThrow();

        expect(function(){
            StylusTemplate = require('../lib/template/StylusTemplate.js').StylusTemplate;
        }).not.toThrow();

        expect(function(){
            Template = require('../lib/template/Template.js').Template;
        }).not.toThrow();
        
        expect(function(){
            JsonRpcAbstract = require('../lib/core/jsonrpc/JsonRpcAbstract.js').JsonRpcAbstract;
        }).not.toThrow();
        
        expect(function(){
            Class = require('../lib/mootools/mootools-node.js').Class;
        }).not.toThrow();
    });
    
    it("createTemplateStatic", function(){
        //container
        var container = TemplateManager.createTemplate('container', configuration.template);
        expect(container).toBeInstanceOf(JqTemplate);
        expect(container).toBeInstanceOf(Template);
        
        expect(function(){
            TemplateManager.createTemplate('container', configuration.unresolved);
        }).toThrow(TemplateManager.Error.UNRESOLVED_TYPE);
    });
    
    it("creatingMockJsonRpc", function(){
        expect(function(){
            JsonRpcMock = function(){
                this.Extends = JsonRpcAbstract;
                
                this._data;
                
                this.initialize = function(data){
                    this._data = data;
                };
                
                this.toJson = function(){
                    return {data: this._data};
                };
            };
            
            JsonRpcMock = new Class(new JsonRpcMock());
        }).not.toThrow();
    });
    
    it("creatingTestTemplate", function(){
        expect(function(){
            TestTemplate = function(){
                this.Extends = Template;
                
                this.render = function(data){
                    this.setRenderedData("eloszka");
                };
            };

            TestTemplate = new Class(new TestTemplate());
        }).not.toThrow();
    });
    
    it("addRemoveTemplateArguments", function(){
        var tm = new TemplateManager();
        
        expect(function(){ tm.addTemplate(); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate(null); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate(undefined); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate(123); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate("eloszka"); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate([]); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate({}); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.addTemplate(new (function(){})); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        
        expect(function(){ tm.removeTemplate(); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate(null); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate(undefined); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate(123); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate("eloszka"); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate([]); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate({}); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
        expect(function(){ tm.removeTemplate(new (function(){})); }).toThrow(TemplateManager.Error.TEMPLATE_HAS_TO_INHERIT);
    });
    
    it("add/remove template", function(){


        var template = new TestTemplate('test', {template: null});
        
        var tm = new TemplateManager();
        tm.addTemplate(template);
        
        expect(template.getTemplateManager()).toBeInstanceOf(TemplateManager);
        expect(template.getTemplateManager()).toBe(tm);
        expect(template.hasEventListener(Template.RENDERED)).toBeTruthy();
        
        expect(tm.removeTemplate(template)).toBeTruthy();
        expect(tm.getTemplateByName('test')).toBeNull();
        
        var template2 = new TestTemplate('test', {template: null});
        expect(tm.removeTemplate(template2)).toBeFalsy();
    });
    
    it("getTemplateByNameArguments", function(){
        var tm = new TemplateManager();

        expect(function(){ tm.getTemplateByName(); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        expect(function(){ tm.getTemplateByName(null); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        expect(function(){ tm.getTemplateByName(undefined); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        expect(function(){ tm.getTemplateByName(123); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        expect(function(){ tm.getTemplateByName([]); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        expect(function(){ tm.getTemplateByName({}); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        expect(function(){ tm.getTemplateByName(new (function(){})); }).toThrow(TemplateManager.Error.NAME_MUST_BE_A_STRING);
        
        expect(function(){ tm.getTemplateByName("eloszka"); }).not.toThrow();
    });
    
    it("getTemplateByName", function(){
        var test1 = new TestTemplate('test1', {template: 1});
        var test2 = new TestTemplate('test2', {template: 2});
        var test3 = new TestTemplate('test3', {template: 3});
        var test4 = new TestTemplate('test4', {template: 4});
        
        var tm = new TemplateManager();
        tm.addTemplate(test1);
        tm.addTemplate(test2);
        tm.addTemplate(test3);
        tm.addTemplate(test4);
        
        expect(tm.getTemplateByName('test2')).toBe(test2);
        expect(tm.getTemplateByName('test2')).toBeInstanceOf(TestTemplate);
        
        expect(tm.getTemplateByName('test4')).toBe(test4);
        expect(tm.getTemplateByName('test4')).toBeInstanceOf(TestTemplate);
    });
    
    it("getTemplatesContent", function(){
        var tm, template, templates = {}, responseEvent = false;
        runs(function(){
            var x = 5;

            tm = new TemplateManager();
            tm.addEventListener(TemplateManager.RENDERED, function(e){ responseEvent = true; });
            
            for(var i = 0; i < x; i++){
                template = new TestTemplate('test' + i, {template: i});
                tm.addTemplate(template);
                template.dispatchRender();
                templates['test' + i] = template.getContent();
            }
            
        });
        
        waitsFor(function(){
            return responseEvent == true;
        }, 1000);

        runs(function(){
            expect(tm.getTemplatesContent()).toEqual(templates);
        });
        
    });
});