describe('JqTemplate', function(){

    var jqtpl = null;
    var JqTemplate = null;
    var Template = null;
    var tpl = null;

    var tempConsoleLog;
    beforeEach(function(){
       tempConsoleLog = console.log;
       console.log = function(text){};
    });
    afterEach(function(){
        console.log = tempConsoleLog;
    });

    it('can load jqtpl module', function(){
        expect(function(){
            jqtpl = require('jqtpl');
        }).not.toThrow();
    });

    it('can load JqTemplate Class', function(){
        expect(function(){
            JqTemplate = require('../../lib/template/JqTemplate.js').JqTemplate;
        }).not.toThrow();
    });

    it('can load Template Class', function(){
        // sprawdzam czy moge wczytac klase Template zeby sprawdzic kolejnego
        // expecta ponizej
        expect(function(){
            Template = require('../../lib/template/Template.js').Template;
        }).not.toThrow();
    });

    it('proper inheritance', function(){
        // sprawdzam czy JqTpl poprawnie dziedziczy
        tpl = new JqTemplate('TplName', {'template':'template_error.tpl'});
        expect(tpl instanceof Template).toBe(true);
    });

    it('template doesn\'t exists', function(){
        
        // przekazano template ktory nie istnieje
        expect(function(){
            tpl = new JqTemplate('TplName', {'template':'template_error.tpl'});
            tpl.render({});
        }).toThrow("Template: template_error.tpl doesn't exist");

    });

    it('template exists', function(){

        var tplContents = '<html></html>';
        jqtpl.template('tpl_01', tplContents);

        tpl = new JqTemplate('TplName', {'template':'tpl_01'});
        tpl.render();
        
        expect(tpl.getContent()).toEqual({result:tplContents});

    });

    it('template with simple data', function(){

        var tplContents = '<title>${data}</title>';
        jqtpl.template('tpl_02', tplContents);

        tpl = new JqTemplate('TplName', {'template':'tpl_02'});
        tpl.render('Title');

        expect(tpl.getContent()).toEqual({result:'<title>Title</title>'});

    });

    it('template with structured data', function(){

        var tplContents = '<title>${data.body[0]}</title><h1>${data.body[1]}</h1>';
        jqtpl.template('tpl_03', tplContents);

        tpl = new JqTemplate('TplName', {'template':'tpl_03'});
        tpl.render({body: ['title', 'header']});

        expect(tpl.getContent()).toEqual({result:'<title>title</title><h1>header</h1>'});

    });

    it('template expects data but  i\'m not sending it', function(){

        var tplContents = '<title>${data.body[0]}</title><h1>${data.body[1]}</h1>';
        jqtpl.template('tpl_04', tplContents);

        tpl = new JqTemplate('TplName', {'template':'tpl_04'});

        expect(function(){
            tpl.render();
        }).toThrow("Cannot read property 'body' of undefined");
        
    });

    it('template expects data but  i\'m not sending wrong data', function(){

        var tplContents = '<title>${data.body[0]}</title><h1>${data.body[1]}</h1>';
        jqtpl.template('tpl_05', tplContents);

        tpl = new JqTemplate('TplName', {'template':'tpl_05'});

        expect(function(){
            tpl.render("misc data");
        }).toThrow("Cannot read property '0' of undefined");

    });


});