xdescribe('StylusTemplate', function(){

    var stylus = null;
    var StylusTemplate = null;
    var Template = null;
    var tpl = null;

    var tempConsoleLog;
    beforeEach(function(){
       tempConsoleLog = console.log;
       console.log = function(text){};
       GLOBAL['stylus'] = {};
    });
    afterEach(function(){
        console.log = tempConsoleLog;
    });

    it('can load stylus module', function(){
        expect(function(){
            stylus = require('stylus');
        }).not.toThrow();
    });

    it('can load StylusTemplate Class', function(){
        expect(function(){
            StylusTemplate = require('../../lib/template/StylusTemplate.js').StylusTemplate;
        }).not.toThrow();
    });

    it('can load Template Class', function(){
        // sprawdzam czy moge wczytac klase Template zeby sprawdzic kolejnego
        // expecta ponizej
        expect(function(){
            Template = require('../../lib/template/Template.js').Template;
        }).not.toThrow();
    });

    it('proper inheritance', function(){
        // sprawdzam czy StylusTemplate poprawnie dziedziczy
        tpl = new StylusTemplate('TplName', {'template':'template_error.tpl'});
        expect(tpl instanceof Template).toBe(true);
    });

    it('template doesn\'t exists', function(){
        // przekazano template ktory nie istnieje
        expect(function(){
            tpl = new StylusTemplate('TplName', {'template':'template_error.tpl'});
            tpl.render({});
        }).toThrow("Template: template_error.tpl doesn't exist");

    });

    it('template exists', function(){
        
        var tplContents = 'body {\n  color: #f00;\n}\n';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        runs(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render();
        });

        waitsFor(function() {
            return tpl.getContent()!==null;
        }, "template never rendered", 500);

        runs(function(){
            expect(tpl.getContent()).toEqual(tplContents);
        });

    });

    it('template without proper indentation', function(){

        var tplContents = 'body {\n  color: #f00;}\n';
        GLOBAL['stylus']['tpl_01'] = tplContents;
        tpl = new StylusTemplate('TplName', {'template':'tpl_01'});

        expect(function(){
            tpl.render();
        }).toThrow();
        
    });

    it('template exists data isn\'t used', function(){

        var tplContents = 'body {\n  color: #f00;\n}\n';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        runs(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render({
                'string': 'string',
                'number': 23
            });
        });

        waitsFor(function() {
            return tpl.getContent()!==null;
        }, "template never rendered", 500);

        runs(function(){
            expect(tpl.getContent()).toEqual(tplContents);
        });

    });

    it('template exists data is good', function(){

        var tplContents = 'body {\n  color: colors;\n}\n';
        var expectedTplContents = 'body {\n  color: #fff;\n}\n';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        runs(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render({data:{
                'colors': '#fff'
            }});
        });

        waitsFor(function() {
            return tpl.getContent()!==null;
        }, "template never rendered", 500);

        runs(function(){
            expect(tpl.getContent()).toEqual(expectedTplContents);
        });

    });

    it('template exists but data isn\'t sent', function(){

        var tplContents = 'body {\n  color: colors;\n}\n';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        runs(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render({data:{
                'colors__': '#fff'
            }});
        });

        waitsFor(function() {
            return tpl.getContent()!==null;
        }, "template never rendered", 500);

        runs(function(){
            expect(tpl.getContent()).toEqual(tplContents);
        });

    });

    xit('template exists, but without ending semicolon', function(){

        var tplContents = 'body{color: #f00}';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        runs(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render();
        });

        waitsFor(function() {
            return tpl.getContent()!==null;
        }, "template never rendered", 500);

        runs(function(){
            expect(tpl.getContent()).toEqual(tplContents);
        });

    });

    xit('template exists, but with syntax error', function(){

        var tplContents = 'body{color: #f00;}\n.error{';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        runs(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render();
        });

        waitsFor(function() {
            return tpl.getContent()!==null;
        }, "template never rendered", 500);

        runs(function(){
            expect(tpl.getContent()).toEqual(tplContents);
        });

    });

    it('template exists, but with syntax error', function(){

        var tplContents = 'body color: red;\n\n}';
        GLOBAL['stylus']['tpl_01'] = tplContents;

        expect(function(){
            tpl = new StylusTemplate('TplName', {'template':'tpl_01'});
            tpl.render();
        }).toThrow();
    });

});